local tab = {}
tab.index = 200
tab.text = "Party/Raid";
tab.value = "party";
tab.columnSort = function (table, rowa, rowb, sortbycol)
	local asc = table.cols[sortbycol].sort == "asc"
	local ka = KeystoneRollCall.tableData[rowa][1]
	local kb = KeystoneRollCall.tableData[rowb][1]
	return KeystoneRollCall:CompareKeystones("name", asc, ka, kb);
end

local function createSpacer(width)
	local label = KeystoneRollCall.AceGUI:Create('Label');
	label:SetText(" ");
	if width <= 0 then
		label:SetFullWidth(true)
	else
		label:SetWidth(width)
	end
	return label
end

local units = {}

local function addUnit(unit)
	for i,u in ipairs(units) do
		if UnitIsUnit(unit,u) then
			return;
		end
	end
	local includeKeyless = KeystoneRollCall.db.global.partyTabShowKeyless
	
	tinsert(units,unit)

	if UnitExists(unit) and UnitIsPlayer(unit) and GetUnitName(unit, true) and UnitClass(unit) then
		local class = UnitClass(unit) or "Warrior"
		local role = UnitGroupRolesAssigned(unit) or ""
		local roleText = role == "DAMAGER" and "DPS" or role == "TANK" and "Tank" or role == "HEALER" and "Healer" or ""
		
		local fullName = GetUnitName(unit, true)
		local nParts =  {strsplit("-",fullName or"")}
		if not nParts[2] or nParts[2] == "" then
			local _,realm = UnitFullName("player");
			fullName = nParts[1] .. "-"..realm
		end
		local keystone = KeystoneRollCall:getKeystone(fullName)
		if includeKeyless or keystone and keystone.level and keystone.level > 0 then
			if not keystone then
				tinsert(KeystoneRollCall.tableData, {
					{['name'] =fullName,['class'] = class,dungeon = -1,level = -1,dummy=true},
					unit,
					KeystoneRollCall:colorRole(roleText,role),
					KeystoneRollCall:colorClass(nParts[1],class),
					KeystoneRollCall:colorClass(class,class),
					"",
					"",
					"", 
					""
				});
			else
				local bestLevelLine = ""
				if keystone.bestLevel  and keystone.bestLevel > 0 and keystone.bestDungeon then
					bestLevelLine = "+" .. KeystoneRollCall:colorLevelDifficulty(keystone.bestLevel,keystone.bestLevel).." " .. C_ChallengeMode.GetMapInfo(keystone.bestDungeon)
				end

				tinsert(KeystoneRollCall.tableData, {
					keystone,
					unit,
					KeystoneRollCall:colorRole(roleText,role),
					KeystoneRollCall:colorClass(nParts[1],class),
					KeystoneRollCall:colorClass(class,class),
					KeystoneRollCall:getIlvlLine(keystone),
					C_ChallengeMode.GetMapInfo(keystone.dungeon),
					keystone.level == 0 and "" or KeystoneRollCall:colorLevelDifficulty(keystone.level,keystone.level),
					bestLevelLine
				});
			end
		end
	end
end

tab.populate = function()
	units = {}
	addUnit("player")
		
	local channel = KeystoneRollCall:getGroupChatChannel();
	if channel == "RAID" then
		for i = 1,40 do
			addUnit("raid"..i)
		end
	elseif channel == "PARTY" then
		for i = 1,5 do
			addUnit("party"..i)
		end
	end
	KeystoneRollCall.scrollTable:SortData();
end

tab.setup = function(container)
	local cols = {{
			['name'] = '',
			['width'] = 1,
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "bestLevel",
			['align'] = 'LEFT'
		},{
			['name'] = '',
			['width'] = 1,
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "bestLevel",
			['align'] = 'LEFT'
		},{
			['name'] = 'Role',
			['width'] = 65,
			['align'] = 'LEFT',
		},{
			['name'] = 'Character',
			['width'] = 85,
			['align'] = 'LEFT', 
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "name",
		},{
			['name'] = 'Class',
			['width'] = 95,
			['align'] = 'LEFT', 
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "class",
		},{
			['name'] = 'ILvl',
			['width'] = 50,
			['align'] = 'CENTER',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "ilvl",
		},{
			['name'] = 'Dungeon',
			['width'] = 160,
			['align'] = 'CENTER',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "dungeon",
		},{
			['name'] = 'Level',
			['width'] = 40,
			['align'] = 'CENTER',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "level",
		},{
			['name'] = "Best Run (This Week)",
			['width'] = 180,
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "bestLevel",
			['align'] = 'LEFT',
		}
	}
	KeystoneRollCall.tableData = {};
	KeystoneRollCall.dummyKeystones = {};
	KeystoneRollCall.scrollTable = KeystoneRollCall.ScrollingTable:CreateST(cols,22);	
	KeystoneRollCall.scrollTable:SetData(KeystoneRollCall.tableData, true);

	KeystoneRollCall.scrollTable:RegisterEvents({
		['OnClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if not KeystoneRollCall.tableData or not KeystoneRollCall.tableData[realrow] then
				return
			end
			if KeystoneRollCall.tableData[realrow][1].dummy then
				SendChatMessage("If you get the addon Keystone Roll-Call we can easily view each other's keystones, as well as keystones from across our guilds.","WHISPER",  "Common",KeystoneRollCall.tableData[realrow][1].name);
			elseif KeystoneRollCall.tableData[realrow][1] then
				local keystone = KeystoneRollCall.tableData[realrow][1];
				if btn == "RightButton" then
					if UnitExists("target") and GetUnitName("target", true) then
						SendChatMessage(KeystoneRollCall:keystoneToString(keystone),"WHISPER", "COMMON",  GetUnitName("target", true));
					end
				else
					SendChatMessage(KeystoneRollCall:keystoneToString(keystone),KeystoneRollCall:getGroupChatChannel() ,  "Common")
				end
			
			end
		end,	
		['OnDoubleClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if KeystoneRollCall.tableData and KeystoneRollCall.tableData[realrow] and not KeystoneRollCall.tableData[realrow][1].dummy then
				SendChatMessage(KeystoneRollCall:keystoneToString(KeystoneRollCall.tableData[realrow][1]),"GUILD" ,  "Common")
			end
		end,	
		['OnEnter'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			if not realrow or not KeystoneRollCall.tableData[realrow] then
				return
				
			end
			if KeystoneRollCall.tableData[realrow][1].dummy then
				KeystoneRollCall.frame:SetStatusText("Click: Suggest Addon")
			else
				KeystoneRollCall.frame:SetStatusText("Left Click: Send to Party   -   Double Click: Send to Guild   -   Right Click: Send to Target")
			end
		end,
		['OnLeave'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			KeystoneRollCall.frame:SetStatusText("")
		end,
	});
	
	
	local tableWrapper = KeystoneRollCall.AceGUI:Create('lib-st'):WrapST(KeystoneRollCall.scrollTable);

	tableWrapper.head_offset = 15;
	container:AddChild(tableWrapper);
	
	container:AddChild(createSpacer(0));
	container:AddChild(createSpacer(50));

	
	
	local broadcastPartyButton = KeystoneRollCall.AceGUI:Create('Button');
	broadcastPartyButton:SetWidth(200);
	broadcastPartyButton:SetText('Broadcast All to Party/Raid');

	broadcastPartyButton:SetCallback('OnClick', function()
		KeystoneRollCall:ReportPartyKeys(KeystoneRollCall:getGroupChatChannel() );
	end);
	container:AddChild(broadcastPartyButton);
	
	
	local broadcastGuildButton = KeystoneRollCall.AceGUI:Create('Button');
	broadcastGuildButton:SetWidth(200);
	broadcastGuildButton:SetText('Broadcast All to Guild');

	broadcastGuildButton:SetCallback('OnClick', function()
		KeystoneRollCall:ReportPartyKeys("GUILD");
	end);
	container:AddChild(broadcastGuildButton);
	
	local whisperTargetButton = KeystoneRollCall.AceGUI:Create('Button');
	whisperTargetButton:SetWidth(200);
	whisperTargetButton:SetText('Whisper All to Target');

	whisperTargetButton:SetCallback('OnClick', function()
		if UnitName("target") then
			KeystoneRollCall:ReportPartyKeys("WHISPER",UnitName("target"));
		end
	end);
	container:AddChild(whisperTargetButton);
end

KeystoneRollCall.tabs[tab.value] = tab;